@@ -9,9 +9,6 @@ typedef struct server server;
 struct connection;
 typedef struct connection connection;
 
-struct h2con;
-typedef struct h2con h2con;
-
 struct plugin;
 typedef struct plugin plugin;
 
@@ -31,8 +28,8 @@ typedef struct fdlog_st log_error_st;
 enum handler_t {
   HANDLER_GO_ON,
   HANDLER_FINISHED,
-  HANDLER_COMEBACK,
   HANDLER_WAIT_FOR_EVENT,
+  HANDLER_COMEBACK,
   HANDLER_ERROR
 };
 typedef enum handler_t handler_t;
