@@ -220,6 +220,9 @@ chunk_file_preadv2_flags (chunk *c)
         if (c->file.is_temp)
             c->file.flagmask = ~RWF_NOWAIT;
       #endif
+      #ifdef __gnu_hurd__ /*RWF_NOWAIT not currently(?) supported on GNU/Hurd*/
+        c->file.flagmask = ~RWF_NOWAIT;
+      #endif
     }
     return (RWF_NOWAIT & c->file.flagmask);
 }
@@ -248,6 +251,12 @@ chunk_file_pread_chunk (chunk *c, void *buf, size_t count)
          * to check EINTR. (sigaction() expected to be present with preadv2())
          * Callers should check c->file.busy before propagating error. */
         int errnum = errno;
+      #ifdef __gnu_hurd__ /*RWF_NOWAIT not currently(?) supported on GNU/Hurd*/
+      #if defined(ENOTSUP) && ENOTSUP != EOPNOTSUPP
+        if (errnum == ENOTSUP)
+            errnum = EOPNOTSUPP;
+      #endif
+      #endif
         if (errnum == EOPNOTSUPP) {  /* WTH?  tmpfs not supported ?!?! */
             c->file.flagmask = ~RWF_NOWAIT;
             return chunk_file_pread_chunk(c, buf, count);/*(tail recurse once)*/
@@ -705,7 +714,7 @@ char * chunkqueue_get_memory(chunkqueue * const restrict cq, size_t * const rest
 	}
 
 	/* allocate new chunk */
-	b = chunkqueue_append_buffer_open_sz(cq, sz);
+	b = chunkqueue_append_buffer_open_sz(cq, sz+1);
 	*len = buffer_string_space(b);
 	return b->ptr;
 }
