@@ -78,7 +78,12 @@ int pcre_keyvalue_buffer_append(log_error_st *errh, pcre_keyvalue_buffer *kvb, c
 
 	if (pcre_jit) {
 		errcode = pcre2_jit_compile(kv->code, PCRE2_JIT_COMPLETE);
-		if (0 != errcode && errcode != PCRE2_ERROR_JIT_BADOPTION) {
+		if (0 != errcode && errcode != PCRE2_ERROR_JIT_BADOPTION
+		    && (errcode != PCRE2_ERROR_NOMEMORY
+		        #ifdef PCRE2_JIT_TEST_ALLOC
+		        || 0 == pcre2_jit_compile(NULL, PCRE2_JIT_TEST_ALLOC)
+		        #endif
+		       )) {
 			pcre2_get_error_message(errcode, errbuf, sizeof(errbuf));
 			log_error(errh, __FILE__, __LINE__,
 			  "pcre2_jit_compile: %s, regex: %s", (char *)errbuf, key->ptr);
@@ -265,7 +270,7 @@ static int pcre_keyvalue_buffer_subst_ext(buffer *b, const char *pattern, const
                 p+=6;
             }
             else if (0 == strncmp((const char *)p, "upper:", 6)) {
-                flags |= BURL_TOLOWER;
+                flags |= BURL_TOUPPER;
                 p+=6;
             }
             else { /* skip unrecognized to... */
@@ -304,6 +309,19 @@ static int pcre_keyvalue_buffer_subst_ext(buffer *b, const char *pattern, const
                     burl_append(b, BUF_PTR_LEN(burl->query), flags);
                 p+=5;
             }
+            else if (0 == strncmp((const char *)p, "authority.noport}", 17)) {
+                /*(note: specific to authority;
+                 * not implemented as "noport:" BURL_* token)*/
+                if (burl->authority) {
+                    const char * const colon = strrchr(burl->authority->ptr, ':');
+                    uint32_t len = buffer_clen(burl->authority);
+                    const char * const ptr = burl->authority->ptr;
+                    if (colon && ptr[len-1] != ']')
+                        len = (uint32_t)(colon - ptr);
+                    burl_append(b, ptr, len, flags);
+                }
+                p+=16;
+            }
             else { /* skip unrecognized url.* */
                 p = (const unsigned char *)strchr((const char *)p, '}');
                 if (NULL == p) return -1;
