@@ -549,7 +549,7 @@ static handler_t mod_auth_require_parse_array(const array *value, array * const
 				dauth->require->algorithm = algorithm;
 				dauth->require->realm = realm;
 				dauth->require->nonce_secret = nonce_secret; /*(NULL is ok)*/
-				dauth->require->userhash = config_plugin_value_tobool(userhash, 0);
+				dauth->require->userhash = config_plugin_value_to_bool(userhash, 0);
 				if (!mod_auth_require_parse(dauth->require, require, errh)) {
 					dauth->fn->free((data_unset *)dauth);
 					return HANDLER_ERROR;
@@ -988,14 +988,14 @@ mod_auth_digest_mutate (http_auth_info_t * const ai, const http_auth_digest_para
 
 
 static void
-mod_auth_append_nonce (buffer *b, unix_time64_t cur_ts, const struct http_auth_require_t *require, int dalgo, int *rndptr)
+mod_auth_append_nonce (buffer *b, unix_time64_t cur_ts, const struct http_auth_require_t *require, int dalgo, unsigned int *rndptr)
 {
     buffer_append_uint_hex(b, (uintmax_t)cur_ts);
     buffer_append_char(b, ':');
     const buffer * const nonce_secret = require->nonce_secret;
-    int rnd;
+    unsigned int rnd;
     if (NULL == nonce_secret)
-        rnd = rndptr ? *rndptr : li_rand_pseudo();
+        rnd = rndptr ? *rndptr : (unsigned int)li_rand_pseudo();
     else { /*(do not directly expose random number generator single value)*/
         rndptr
           ? (void)(rnd = *rndptr)
@@ -1454,7 +1454,7 @@ mod_auth_digest_validate_nonce (request_st * const r, const struct http_auth_req
      * that is highly unlikely within a 10 min (moving) window of valid
      * time relative to current time (now).
      * If it is desired to validate that nonces were generated by server,
-     * then specify auth.require = ( ... => ( "secret" => "..." ) )
+     * then specify auth.require = ( ... => ( "nonce-secret" => "..." ) )
      * When secret is specified, then instead of nanoseconds, the random
      * data value (included for unique nonces) will be exposed in the nonce
      * along with the timestamp, and the additional secret will be used to
@@ -1466,7 +1466,7 @@ mod_auth_digest_validate_nonce (request_st * const r, const struct http_auth_req
         ts = (unix_time64_t)((uint64_t)ts << 4) | hex2int(nonce[i]);
 
     const unix_time64_t cur_ts = log_epoch_secs;
-    if (nonce[i] != ':' || ts > cur_ts || cur_ts - ts > 600) { /*(10 mins)*/
+    if (nonce[i++] != ':' || ts > cur_ts || cur_ts - ts > 600) { /*(10 mins)*/
         /* nonce is stale; have client regenerate digest */
         return mod_auth_send_401_unauthorized_digest(r, require, ai->dalgo);
     }
@@ -1488,7 +1488,7 @@ mod_auth_digest_validate_nonce (request_st * const r, const struct http_auth_req
         }
         buffer * const tb = r->tmp_buf;
         buffer_clear(tb);
-        mod_auth_append_nonce(tb, cur_ts, require, ai->dalgo, (int *)&rnd);
+        mod_auth_append_nonce(tb, ts, require, ai->dalgo, &rnd);
         if (!buffer_eq_slen(tb, dp->ptr[e_nonce], dp->len[e_nonce])) {
             /* nonce not generated using current require->nonce_secret */
             log_error(r->conf.errh, __FILE__, __LINE__,
@@ -1538,9 +1538,24 @@ mod_auth_check_digest (request_st * const r, void *p_d, const struct http_auth_r
     if (__builtin_expect( (HANDLER_GO_ON != rc), 0))
         return rc;
 
+    int eq;
+    unsigned char digcpy[sizeof(ai.digest)];
+    if (r->h2_connect_ext)
+        memcpy(digcpy, ai.digest, ai.dlen);
+
     mod_auth_digest_mutate(&ai, &dp, http_method_buf(r->http_method));
 
-    if (!ck_memeq_const_time_fixed_len(dp.rdigest, ai.digest, ai.dlen)) {
+    eq = ck_memeq_const_time_fixed_len(dp.rdigest, ai.digest, ai.dlen);
+    if (r->h2_connect_ext) {
+        if (!eq) {
+            memcpy(ai.digest, digcpy, ai.dlen);
+            mod_auth_digest_mutate(&ai, &dp, http_method_buf(HTTP_METHOD_GET));
+            eq = ck_memeq_const_time_fixed_len(dp.rdigest, ai.digest, ai.dlen);
+        }
+        ck_memzero(digcpy, ai.dlen);
+    }
+
+    if (!eq) {
         /*ck_memzero(ai.digest, ai.dlen);*//*skip clear since mutated*/
         /* digest not ok */
         log_error(r->conf.errh, __FILE__, __LINE__,
