@@ -693,7 +693,7 @@ static handler_t mod_authn_ldap_memberOf(log_error_st *errh, plugin_config *s, c
 
     plugin_config_ldap * const ldc = s->ldc;
     for (size_t i = 0; i < groups->used; ++i) {
-        const char *base = groups->data[i]->key.ptr;
+        const char *base = ((data_string *)groups->data[i])->value.ptr;
         LDAPMessage *lm = mod_authn_ldap_search(errh, ldc, base, filter->ptr);
         if (NULL != lm) {
             int count = ldap_count_entries(ldc->ldap, lm);
@@ -816,6 +816,7 @@ static handler_t mod_authn_ldap_basic(request_st * const r, void *p_d, const htt
 
 
 __attribute_cold__
+__declspec_dllexport__
 int mod_authn_ldap_plugin_init(plugin *p);
 int mod_authn_ldap_plugin_init(plugin *p) {
     p->version     = LIGHTTPD_VERSION_ID;
