@@ -7,13 +7,13 @@
 
 #include "plugin.h"
 #include <sys/types.h>
-#include <sys/stat.h>
+#include "sys-stat.h"
 #include "sys-time.h"
+#include "sys-unistd.h" /* <unistd.h> */
 
 #include <fcntl.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 #include <errno.h>
 
 typedef struct {
@@ -432,6 +432,7 @@ REQUESTDONE_FUNC(mod_rrd_account) {
 
 
 __attribute_cold__
+__declspec_dllexport__
 int mod_rrdtool_plugin_init(plugin *p);
 int mod_rrdtool_plugin_init(plugin *p) {
 	p->version     = LIGHTTPD_VERSION_ID;
