@@ -9,7 +9,15 @@
 #include <string.h>
 #include <stdlib.h>
 
+#ifdef __has_include
+#if __has_include(<dbi/dbi.h>)
 #include <dbi/dbi.h>
+#else
+#include <dbi.h>
+#endif
+#else
+#include <dbi/dbi.h>
+#endif
 
 #include "mod_vhostdb_api.h"
 #include "base.h"
@@ -54,7 +62,8 @@ static void mod_vhostdb_dbi_error_callback (dbi_conn dbconn, void *vdata)
 
     while (++dbconf->reconnect_count <= 3) { /* retry */
         if (0 == dbi_conn_connect(dbconn)) {
-            fdevent_setfd_cloexec(dbi_conn_get_socket(dbconn));
+            /* _WIN32: ok if SOCKET (unsigned long long) actually <= INT_MAX */
+            (void)fdevent_socket_set_cloexec(dbi_conn_get_socket(dbconn));
             return;
         }
     }
@@ -328,6 +337,7 @@ SETDEFAULTS_FUNC(mod_vhostdb_set_defaults) {
 
 
 __attribute_cold__
+__declspec_dllexport__
 int mod_vhostdb_dbi_plugin_init (plugin *p);
 int mod_vhostdb_dbi_plugin_init (plugin *p)
 {
