@@ -114,8 +114,10 @@ static handler_t network_server_handle_fdevent(void *context, int revents) {
       #else
         switch (errno) {
           case EAGAIN:
+         #ifdef EWOULDBLOCK
          #if EWOULDBLOCK != EAGAIN
           case EWOULDBLOCK:
+         #endif
          #endif
           case EINTR:
           case ECONNABORTED:
@@ -859,7 +861,7 @@ int network_init(server *srv, int stdin_fd) {
     if (!config_plugin_values_init(srv, p, cpk, "network"))
         return HANDLER_ERROR;
 
-    p->defaults.listen_backlog = 1024;
+    p->defaults.listen_backlog = SOMAXCONN > 1024 ? SOMAXCONN : 1024;
     p->defaults.defer_accept = 0;
     p->defaults.use_ipv6 = 0;
     p->defaults.set_v6only = 1;
