@@ -105,6 +105,11 @@ static int network_write_error(int fd, log_error_st *errh) {
   #else
     switch (errno) {
       case EAGAIN:
+     #ifdef EWOULDBLOCK
+     #if EWOULDBLOCK != EAGAIN
+      case EWOULDBLOCK:
+     #endif
+     #endif
       case EINTR:
         return -3;
       case EPIPE:
@@ -291,7 +296,7 @@ static int network_writev_mem_chunks(const int fd, chunkqueue * const cq, off_t
 
   #ifdef _WIN32
     DWORD dw;
-    ssize_t wr = WSASend(fd, chunks, num_chunks, &dw, 0, NULL, NULL);
+    ssize_t wr = WSASend(fd, chunks, (DWORD)num_chunks, &dw, 0, NULL, NULL);
     if (0 == wr) wr = (ssize_t)dw;
   #else
     ssize_t wr = writev(fd, chunks, num_chunks);
@@ -476,6 +481,11 @@ static int network_write_file_chunk_sendfile(const int fd, chunkqueue * const cq
             break; /* try again later */
          #endif
           case EAGAIN:
+         #ifdef EWOULDBLOCK
+         #if EWOULDBLOCK != EAGAIN
+          case EWOULDBLOCK:
+         #endif
+         #endif
           case EINTR:
             break; /* try again later */
           case EPIPE:
